"""
Huracan engine components
"""

from huracan.components.static import intake, inlet, nozzle
from huracan.components.rotary import fan, prop, propfan, compressor, turbine
from huracan.components.combustion import combustion_chamber, afterburner
from huracan.components.electrical import power_plant
